package com.chilunyc.fumao.controller.error;

import com.chilunyc.fumao.common.config.ApplicationProperty;
import com.chilunyc.fumao.common.exception.SystemException;
import com.chilunyc.fumao.common.log.model.EventTarget;
import com.chilunyc.fumao.common.log.model.EventType;
import com.chilunyc.fumao.common.log.service.EventService;
import com.chilunyc.fumao.common.context.RequestHolder;
import com.chilunyc.fumao.common.response.JsonResponse;
import lombok.extern.log4j.Log4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.context.NoSuchMessageException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.util.ContentCachingRequestWrapper;

import java.util.Arrays;
import java.util.List;
import java.util.Locale;

/**
 *
 */
@Log4j
@ControllerAdvice
class ExceptionController {

    @Autowired
    MessageSource messageSource;

    @Autowired
    EventService eventService;

    @Autowired
    ApplicationProperty applicationProperty;

    /**
     * 业务异常
     *
     * @param request
     * @param ex
     * @return
     */
    @ExceptionHandler(SystemException.class)
    @ResponseStatus(HttpStatus.OK)
    @ResponseBody
    JsonResponse systemExceptionHandler(ContentCachingRequestWrapper request, SystemException ex) {
        String errorCode = ex.getErrorCode();
        if (StringUtils.isEmpty(errorCode)) {
            errorCode = "000001";
        }
        String msg = this.getMessage(errorCode, ex.getParams());
        String requestContent = "";
        try {
            requestContent = IOUtils.toString(request.getContentAsByteArray(), "UTF-8");
        } catch (Exception e) {
        }
        if (applicationProperty.isTest()) {
            log.error(request.getServletPath() + " => " + errorCode + " => " + requestContent + " ==> " + msg, ex);
        } else {
            String errorMsg = request.getServletPath() + " => " + errorCode + " => " + requestContent + " ==> " + msg + " || " + this.getStackJson(ex);
            eventService.add(RequestHolder.get(), EventTarget.None, EventType.Error, null, errorMsg);
        }
        return JsonResponse.fail(errorCode, msg);
    }

    /**
     * 参数验证
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    @ResponseStatus(HttpStatus.OK)
    @ResponseBody
    JsonResponse validationHandler(MethodArgumentNotValidException ex) {
        List<FieldError> fieldErrors = ex.getBindingResult().getFieldErrors();
        FieldError fr = fieldErrors.get(fieldErrors.size() - 1);
        return JsonResponse.fail("000003", fr.getDefaultMessage());
    }

    /**
     * 参数验证
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(BindException.class)
    @ResponseStatus(HttpStatus.OK)
    @ResponseBody
    JsonResponse validationHandler(BindException ex) {
        List<FieldError> fieldErrors = ex.getBindingResult().getFieldErrors();
        FieldError fr = fieldErrors.get(fieldErrors.size() - 1);
        return JsonResponse.fail("000003", fr.getDefaultMessage());
    }

    /**
     * 数据不存在
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(EmptyResultDataAccessException.class)
    @ResponseStatus(HttpStatus.OK)
    @ResponseBody
    JsonResponse noDataHandler(EmptyResultDataAccessException ex) {
        log.error(ex.getMessage(), ex);
        return JsonResponse.fail("000004", this.getMessage("000004", null));
    }

    /**
     * 错误统一处理
     *
     * @param request
     * @param ex
     * @return
     * @throws Exception
     */
    @ExceptionHandler
    @ResponseBody
    JsonResponse defaultExceptionHandler(ContentCachingRequestWrapper request, Exception ex) throws Exception {
        String requestContent = "";
        try {
            requestContent = IOUtils.toString(request.getContentAsByteArray(), "UTF-8");
        } catch (Exception e) {
        }
        if (applicationProperty.isTest()) {
            log.error(request.getServletPath() + " => " + requestContent + " ==> " + ex.getMessage(), ex);
            return JsonResponse.fail(ex.getMessage());
        } else {
            String errorMsg = request.getServletPath() + " => " + requestContent + " ==> " + ex.getMessage() + " || " + this.getStackJson(ex);
            eventService.add(RequestHolder.get(), EventTarget.None, EventType.Error, null, errorMsg);
            return JsonResponse.fail();
        }
    }

    protected String getMessage(String code, Object[] params) {
        try {
            return messageSource.getMessage(code, params, Locale.CHINA);
        } catch (NoSuchMessageException e) {
            return code;
        }
    }

    private String getStackJson(Throwable e) {
        String result = "";
        if (null != e) {
            StackTraceElement[] s = e.getStackTrace();
            if (null != s) {
                if (s.length > 10) {
                    s = Arrays.copyOfRange(s, 0, 9);
                }
                result = "\n";
                for (StackTraceElement ste : s) {
                    result += ("\tat " + ste + "\n");
                }
                if (null != e.getCause()) {
                    result += "Cause By:" + StringUtils.trimToEmpty(e.getCause().getMessage());
                    result += getStackJson(e.getCause());
                }
            }
        }
        return result;
    }


}
